@@ -44,8 +44,31 @@
 #  include <locale.h>
 # endif
 
-static const char *
-stringprep_locale_charset_slow (void)
+# ifdef _LIBC
+#  define stringprep_locale_charset() nl_langinfo (CODESET)
+# else
+/**
+ * stringprep_locale_charset:
+ *
+ * Find out current locale charset.  The function respect the CHARSET
+ * environment variable, but typically uses nl_langinfo(CODESET) when
+ * it is supported.  It fall back on "ASCII" if CHARSET isn't set and
+ * nl_langinfo isn't supported or return anything.
+ *
+ * Note that this function return the application's locale's preferred
+ * charset (or thread's locale's preffered charset, if your system
+ * support thread-specific locales).  It does not return what the
+ * system may be using.  Thus, if you receive data from external
+ * sources you cannot in general use this function to guess what
+ * charset it is encoded in.  Use stringprep_convert from the external
+ * representation into the charset returned by this function, to have
+ * data in the locale encoding.
+ *
+ * Return value: Return the character set used by the current locale.
+ *   It will never return NULL, but use "ASCII" as a fallback.
+ **/
+const char *
+stringprep_locale_charset (void)
 {
   const char *charset = getenv ("CHARSET");	/* flawfinder: ignore */
 
@@ -53,55 +76,15 @@ stringprep_locale_charset_slow (void)
     return charset;
 
 # ifdef LOCALE_WORKS
-  {
-    char *p;
+  charset = nl_langinfo (CODESET);
 
-    p = setlocale (LC_CTYPE, NULL);
-    setlocale (LC_CTYPE, "");
-
-    charset = nl_langinfo (CODESET);
-
-    setlocale (LC_CTYPE, p);
-
-    if (charset && *charset)
-      return charset;
-  }
+  if (charset && *charset)
+    return charset;
 # endif
 
   return "ASCII";
 }
-
-static const char *stringprep_locale_charset_cache = NULL;
-
-/**
- * stringprep_locale_charset:
- *
- * Find out system locale charset.
- *
- * Note that this function return what it believe the SYSTEM is using
- * as a locale, not what locale the program is currently in (modified,
- * e.g., by a setlocale(LC_CTYPE, "ISO-8859-1")).  The reason is that
- * data read from argv[], stdin etc comes from the system, and is more
- * likely to be encoded using the system locale than the program
- * locale.
- *
- * You can set the environment variable CHARSET to override the value
- * returned.  Note that this function caches the result, so you will
- * have to modify CHARSET before calling (even indirectly) any
- * stringprep functions, e.g., by setting it when invoking the
- * application.
- *
- * Return value: Return the character set used by the system locale.
- *   It will never return NULL, but use "ASCII" as a fallback.
- **/
-const char *
-stringprep_locale_charset (void)
-{
-  if (!stringprep_locale_charset_cache)
-    stringprep_locale_charset_cache = stringprep_locale_charset_slow ();
-
-  return stringprep_locale_charset_cache;
-}
+#endif
 
 /**
  * stringprep_convert:
@@ -122,13 +105,12 @@ stringprep_convert (const char *str,
   iconv_t cd;
   char *dest;
   char *outp;
-  char *p, *startp;
+  ICONV_CONST char *p;
   size_t inbytes_remaining;
   size_t outbytes_remaining;
   size_t err;
   size_t outbuf_size;
   int have_error = 0;
-  int len;
 
   if (strcmp (to_codeset, from_codeset) == 0)
     {
@@ -136,8 +118,7 @@ stringprep_convert (const char *str,
       p = malloc (strlen (str) + 1);
       if (!p)
 	return NULL;
-      strcpy (p, str);
-      return p;
+      return strcpy (p, str);
     }
 
   cd = iconv_open (to_codeset, from_codeset);
@@ -145,17 +126,16 @@ stringprep_convert (const char *str,
   if (cd == (iconv_t) - 1)
     return NULL;
 
-  p = (char *) malloc (strlen (str) + 1);
-  strcpy (p, str);
-  if (p == NULL)
-    return NULL;
-  len = strlen (p);
-  startp = p;
-  inbytes_remaining = len;
-  outbuf_size = len + 1;	/* + 1 for nul in case len == 1 */
+  p = (ICONV_CONST char *) str;
+
+  inbytes_remaining = strlen (p);
+  /* Guess the maximum length the output string can have.  */
+  outbuf_size = (inbytes_remaining + 1) * 5;
 
-  outbytes_remaining = outbuf_size - 1;	/* -1 for nul */
   outp = dest = malloc (outbuf_size);
+  if (dest == NULL)
+    goto out;
+  outbytes_remaining = outbuf_size - 1;	/* -1 for NUL */
 
 again:
 
@@ -173,12 +153,19 @@ again:
 	case E2BIG:
 	  {
 	    size_t used = outp - dest;
+	    char *newdest;
 
 	    outbuf_size *= 2;
-	    dest = realloc (dest, outbuf_size);
+	    newdest = realloc (dest, outbuf_size);
+	    if (newdest == NULL)
+	      {
+		have_error = 1;
+		goto out;
+	      }
+	    dest = newdest;
 
 	    outp = dest + used;
-	    outbytes_remaining = outbuf_size - used - 1;	/* -1 for nul */
+	    outbytes_remaining = outbuf_size - used - 1; /* -1 for NUL */
 
 	    goto again;
 	  }
@@ -196,12 +183,10 @@ again:
 
   *outp = '\0';
 
-  if ((p - startp) != len)
+  if (*p != '\0')
     have_error = 1;
 
-
-  free (startp);
-
+ out:
   iconv_close (cd);
 
   if (have_error)
