@@ -1,4 +1,4 @@
-/* Copyright (C) 1996,97,98,99,2000,01,02 Free Software Foundation, Inc.
+  /* Copyright (C) 1996-2002, 2003, 2004 Free Software Foundation, Inc.
    This file is part of the GNU C Library.
 
    The GNU C Library is free software; you can redistribute it and/or
@@ -104,102 +104,146 @@ struct hostent
 };
 
 /* Open host data base files and mark them as staying open even after
-   a later search if STAY_OPEN is non-zero.  */
-extern void sethostent (int __stay_open) __THROW;
+   a later search if STAY_OPEN is non-zero.
 
-/* Close host data base files and clear `stay open' flag.  */
-extern void endhostent (void) __THROW;
+   This function is a possible cancellation point and therefore not
+   marked with __THROW.  */
+extern void sethostent (int __stay_open);
+
+/* Close host data base files and clear `stay open' flag.
+
+   This function is a possible cancellation point and therefore not
+   marked with __THROW.  */
+extern void endhostent (void);
 
 /* Get next entry from host data base file.  Open data base if
-   necessary.  */
-extern struct hostent *gethostent (void) __THROW;
+   necessary.
+
+   This function is a possible cancellation point and therefore not
+   marked with __THROW.  */
+extern struct hostent *gethostent (void);
 
 /* Return entry from host data base which address match ADDR with
-   length LEN and type TYPE.  */
-extern struct hostent *gethostbyaddr (__const void *__addr, __socklen_t __len,
-				      int __type) __THROW;
+   length LEN and type TYPE.
 
-/* Return entry from host data base for host with NAME.  */
-extern struct hostent *gethostbyname (__const char *__name) __THROW;
+   This function is a possible cancellation point and therefore not
+   marked with __THROW.  */
+extern struct hostent *gethostbyaddr (__const void *__addr, __socklen_t __len,
+				      int __type);
+
+/* Return entry from host data base for host with NAME.
+
+   This function is a possible cancellation point and therefore not
+   marked with __THROW.  */
+extern struct hostent *gethostbyname (__const char *__name);
 
 #ifdef __USE_MISC
 /* Return entry from host data base for host with NAME.  AF must be
    set to the address type which is `AF_INET' for IPv4 or `AF_INET6'
-   for IPv6.  */
-extern struct hostent *gethostbyname2 (__const char *__name, int __af) __THROW;
+   for IPv6.
+
+   This function is not part of POSIX and therefore no official
+   cancellation point.  But due to similarity with an POSIX interface
+   or due to the implementation it is a cancellation point and
+   therefore not marked with __THROW.  */
+extern struct hostent *gethostbyname2 (__const char *__name, int __af);
 
 /* Reentrant versions of the functions above.  The additional
    arguments specify a buffer of BUFLEN starting at BUF.  The last
    argument is a pointer to a variable which gets the value which
    would be stored in the global variable `herrno' by the
-   non-reentrant functions.  */
+   non-reentrant functions.
+
+   These functions are not part of POSIX and therefore no official
+   cancellation point.  But due to similarity with an POSIX interface
+   or due to the implementation they are cancellation points and
+   therefore not marked with __THROW.  */
 extern int gethostent_r (struct hostent *__restrict __result_buf,
 			 char *__restrict __buf, size_t __buflen,
 			 struct hostent **__restrict __result,
-			 int *__restrict __h_errnop) __THROW;
+			 int *__restrict __h_errnop);
 
 extern int gethostbyaddr_r (__const void *__restrict __addr, __socklen_t __len,
 			    int __type,
 			    struct hostent *__restrict __result_buf,
 			    char *__restrict __buf, size_t __buflen,
 			    struct hostent **__restrict __result,
-			    int *__restrict __h_errnop) __THROW;
+			    int *__restrict __h_errnop);
 
 extern int gethostbyname_r (__const char *__restrict __name,
 			    struct hostent *__restrict __result_buf,
 			    char *__restrict __buf, size_t __buflen,
 			    struct hostent **__restrict __result,
-			    int *__restrict __h_errnop) __THROW;
+			    int *__restrict __h_errnop);
 
 extern int gethostbyname2_r (__const char *__restrict __name, int __af,
 			     struct hostent *__restrict __result_buf,
 			     char *__restrict __buf, size_t __buflen,
 			     struct hostent **__restrict __result,
-			     int *__restrict __h_errnop) __THROW;
+			     int *__restrict __h_errnop);
 #endif	/* misc */
 
 
 /* Open network data base files and mark them as staying open even
-   after a later search if STAY_OPEN is non-zero.  */
-extern void setnetent (int __stay_open) __THROW;
+   after a later search if STAY_OPEN is non-zero.
 
-/* Close network data base files and clear `stay open' flag.  */
-extern void endnetent (void) __THROW;
+   This function is a possible cancellation point and therefore not
+   marked with __THROW.  */
+extern void setnetent (int __stay_open);
+
+/* Close network data base files and clear `stay open' flag.
+
+   This function is a possible cancellation point and therefore not
+   marked with __THROW.  */
+extern void endnetent (void);
 
 /* Get next entry from network data base file.  Open data base if
-   necessary.  */
-extern struct netent *getnetent (void) __THROW;
+   necessary.
+
+   This function is a possible cancellation point and therefore not
+   marked with __THROW.  */
+extern struct netent *getnetent (void);
 
 /* Return entry from network data base which address match NET and
-   type TYPE.  */
-extern struct netent *getnetbyaddr (uint32_t __net, int __type)
-     __THROW;
+   type TYPE.
 
-/* Return entry from network data base for network with NAME.  */
-extern struct netent *getnetbyname (__const char *__name) __THROW;
+   This function is a possible cancellation point and therefore not
+   marked with __THROW.  */
+extern struct netent *getnetbyaddr (uint32_t __net, int __type);
+
+/* Return entry from network data base for network with NAME.
+
+   This function is a possible cancellation point and therefore not
+   marked with __THROW.  */
+extern struct netent *getnetbyname (__const char *__name);
 
 #ifdef	__USE_MISC
 /* Reentrant versions of the functions above.  The additional
    arguments specify a buffer of BUFLEN starting at BUF.  The last
    argument is a pointer to a variable which gets the value which
    would be stored in the global variable `herrno' by the
-   non-reentrant functions.  */
+   non-reentrant functions.
+
+   These functions are not part of POSIX and therefore no official
+   cancellation point.  But due to similarity with an POSIX interface
+   or due to the implementation they are cancellation points and
+   therefore not marked with __THROW.  */
 extern int getnetent_r (struct netent *__restrict __result_buf,
 			char *__restrict __buf, size_t __buflen,
 			struct netent **__restrict __result,
-			int *__restrict __h_errnop) __THROW;
+			int *__restrict __h_errnop);
 
 extern int getnetbyaddr_r (uint32_t __net, int __type,
 			   struct netent *__restrict __result_buf,
 			   char *__restrict __buf, size_t __buflen,
 			   struct netent **__restrict __result,
-			   int *__restrict __h_errnop) __THROW;
+			   int *__restrict __h_errnop);
 
 extern int getnetbyname_r (__const char *__restrict __name,
 			   struct netent *__restrict __result_buf,
 			   char *__restrict __buf, size_t __buflen,
 			   struct netent **__restrict __result,
-			   int *__restrict __h_errnop) __THROW;
+			   int *__restrict __h_errnop);
 #endif	/* misc */
 
 
@@ -213,44 +257,63 @@ struct servent
 };
 
 /* Open service data base files and mark them as staying open even
-   after a later search if STAY_OPEN is non-zero.  */
-extern void setservent (int __stay_open) __THROW;
+   after a later search if STAY_OPEN is non-zero.
 
-/* Close service data base files and clear `stay open' flag.  */
-extern void endservent (void) __THROW;
+   This function is a possible cancellation point and therefore not
+   marked with __THROW.  */
+extern void setservent (int __stay_open);
+
+/* Close service data base files and clear `stay open' flag.
+
+   This function is a possible cancellation point and therefore not
+   marked with __THROW.  */
+extern void endservent (void);
 
 /* Get next entry from service data base file.  Open data base if
-   necessary.  */
-extern struct servent *getservent (void) __THROW;
+   necessary.
+
+   This function is a possible cancellation point and therefore not
+   marked with __THROW.  */
+extern struct servent *getservent (void);
 
 /* Return entry from network data base for network with NAME and
-   protocol PROTO.  */
+   protocol PROTO.
+
+   This function is a possible cancellation point and therefore not
+   marked with __THROW.  */
 extern struct servent *getservbyname (__const char *__name,
-				      __const char *__proto) __THROW;
+				      __const char *__proto);
 
 /* Return entry from service data base which matches port PORT and
-   protocol PROTO.  */
-extern struct servent *getservbyport (int __port, __const char *__proto)
-     __THROW;
+   protocol PROTO.
+
+   This function is a possible cancellation point and therefore not
+   marked with __THROW.  */
+extern struct servent *getservbyport (int __port, __const char *__proto);
 
 
 #ifdef	__USE_MISC
 /* Reentrant versions of the functions above.  The additional
-   arguments specify a buffer of BUFLEN starting at BUF.  */
+   arguments specify a buffer of BUFLEN starting at BUF.
+
+   These functions are not part of POSIX and therefore no official
+   cancellation point.  But due to similarity with an POSIX interface
+   or due to the implementation they are cancellation points and
+   therefore not marked with __THROW.  */
 extern int getservent_r (struct servent *__restrict __result_buf,
 			 char *__restrict __buf, size_t __buflen,
-			 struct servent **__restrict __result) __THROW;
+			 struct servent **__restrict __result);
 
 extern int getservbyname_r (__const char *__restrict __name,
 			    __const char *__restrict __proto,
 			    struct servent *__restrict __result_buf,
 			    char *__restrict __buf, size_t __buflen,
-			    struct servent **__restrict __result) __THROW;
+			    struct servent **__restrict __result);
 
 extern int getservbyport_r (int __port, __const char *__restrict __proto,
 			    struct servent *__restrict __result_buf,
 			    char *__restrict __buf, size_t __buflen,
-			    struct servent **__restrict __result) __THROW;
+			    struct servent **__restrict __result);
 #endif	/* misc */
 
 
@@ -263,64 +326,108 @@ struct protoent
 };
 
 /* Open protocol data base files and mark them as staying open even
-   after a later search if STAY_OPEN is non-zero.  */
-extern void setprotoent (int __stay_open) __THROW;
+   after a later search if STAY_OPEN is non-zero.
 
-/* Close protocol data base files and clear `stay open' flag.  */
-extern void endprotoent (void) __THROW;
+   This function is a possible cancellation point and therefore not
+   marked with __THROW.  */
+extern void setprotoent (int __stay_open);
+
+/* Close protocol data base files and clear `stay open' flag.
+
+   This function is a possible cancellation point and therefore not
+   marked with __THROW.  */
+extern void endprotoent (void);
 
 /* Get next entry from protocol data base file.  Open data base if
-   necessary.  */
-extern struct protoent *getprotoent (void) __THROW;
+   necessary.
 
-/* Return entry from protocol data base for network with NAME.  */
-extern struct protoent *getprotobyname (__const char *__name) __THROW;
+   This function is a possible cancellation point and therefore not
+   marked with __THROW.  */
+extern struct protoent *getprotoent (void);
 
-/* Return entry from protocol data base which number is PROTO.  */
-extern struct protoent *getprotobynumber (int __proto) __THROW;
+/* Return entry from protocol data base for network with NAME.
+
+   This function is a possible cancellation point and therefore not
+   marked with __THROW.  */
+extern struct protoent *getprotobyname (__const char *__name);
+
+/* Return entry from protocol data base which number is PROTO.
+
+   This function is a possible cancellation point and therefore not
+   marked with __THROW.  */
+extern struct protoent *getprotobynumber (int __proto);
 
 
 #ifdef	__USE_MISC
 /* Reentrant versions of the functions above.  The additional
-   arguments specify a buffer of BUFLEN starting at BUF.  */
+   arguments specify a buffer of BUFLEN starting at BUF.
+
+   These functions are not part of POSIX and therefore no official
+   cancellation point.  But due to similarity with an POSIX interface
+   or due to the implementation they are cancellation points and
+   therefore not marked with __THROW.  */
 extern int getprotoent_r (struct protoent *__restrict __result_buf,
 			  char *__restrict __buf, size_t __buflen,
-			  struct protoent **__restrict __result) __THROW;
+			  struct protoent **__restrict __result);
 
 extern int getprotobyname_r (__const char *__restrict __name,
 			     struct protoent *__restrict __result_buf,
 			     char *__restrict __buf, size_t __buflen,
-			     struct protoent **__restrict __result) __THROW;
+			     struct protoent **__restrict __result);
 
 extern int getprotobynumber_r (int __proto,
 			       struct protoent *__restrict __result_buf,
 			       char *__restrict __buf, size_t __buflen,
-			       struct protoent **__restrict __result) __THROW;
-#endif	/* misc */
+			       struct protoent **__restrict __result);
 
 
-/* Establish network group NETGROUP for enumeration.  */
-extern int setnetgrent (__const char *__netgroup) __THROW;
+/* Establish network group NETGROUP for enumeration.
 
-/* Free all space allocated by previous `setnetgrent' call.  */
-extern void endnetgrent (void) __THROW;
+   This function is not part of POSIX and therefore no official
+   cancellation point.  But due to similarity with an POSIX interface
+   or due to the implementation it is a cancellation point and
+   therefore not marked with __THROW.  */
+extern int setnetgrent (__const char *__netgroup);
+
+/* Free all space allocated by previous `setnetgrent' call.
+
+   This function is not part of POSIX and therefore no official
+   cancellation point.  But due to similarity with an POSIX interface
+   or due to the implementation it is a cancellation point and
+   therefore not marked with __THROW.  */
+extern void endnetgrent (void);
 
 /* Get next member of netgroup established by last `setnetgrent' call
-   and return pointers to elements in HOSTP, USERP, and DOMAINP.  */
+   and return pointers to elements in HOSTP, USERP, and DOMAINP.
+
+   This function is not part of POSIX and therefore no official
+   cancellation point.  But due to similarity with an POSIX interface
+   or due to the implementation it is a cancellation point and
+   therefore not marked with __THROW.  */
 extern int getnetgrent (char **__restrict __hostp,
 			char **__restrict __userp,
-			char **__restrict __domainp) __THROW;
+			char **__restrict __domainp);
 
-#ifdef	__USE_MISC
-/* Test whether NETGROUP contains the triple (HOST,USER,DOMAIN).  */
+
+/* Test whether NETGROUP contains the triple (HOST,USER,DOMAIN).
+
+   This function is not part of POSIX and therefore no official
+   cancellation point.  But due to similarity with an POSIX interface
+   or due to the implementation it is a cancellation point and
+   therefore not marked with __THROW.  */
 extern int innetgr (__const char *__netgroup, __const char *__host,
-		    __const char *__user, __const char *domain) __THROW;
+		    __const char *__user, __const char *domain);
 
-/* Reentrant version of `getnetgrent' where result is placed in BUFFER.  */
+/* Reentrant version of `getnetgrent' where result is placed in BUFFER.
+
+   This function is not part of POSIX and therefore no official
+   cancellation point.  But due to similarity with an POSIX interface
+   or due to the implementation it is a cancellation point and
+   therefore not marked with __THROW.  */
 extern int getnetgrent_r (char **__restrict __hostp,
 			  char **__restrict __userp,
 			  char **__restrict __domainp,
-			  char *__restrict __buffer, size_t __buflen) __THROW;
+			  char *__restrict __buffer, size_t __buflen);
 #endif	/* misc */
 
 
@@ -330,60 +437,98 @@ extern int getnetgrent_r (char **__restrict __hostp,
    executed as REMUSER.  In *FD2P the descriptor to the socket for the
    connection is returned.  The caller must have the right to use a
    reserved port.  When the function returns *AHOST contains the
-   official host name.  */
+   official host name.
+
+   This function is not part of POSIX and therefore no official
+   cancellation point.  But due to similarity with an POSIX interface
+   or due to the implementation it is a cancellation point and
+   therefore not marked with __THROW.  */
 extern int rcmd (char **__restrict __ahost, unsigned short int __rport,
 		 __const char *__restrict __locuser,
 		 __const char *__restrict __remuser,
-		 __const char *__restrict __cmd, int *__restrict __fd2p)
-     __THROW;
+		 __const char *__restrict __cmd, int *__restrict __fd2p);
 
 /* This is the equivalent function where the protocol can be selected
-   and which therefore can be used for IPv6.  */
+   and which therefore can be used for IPv6.
+
+   This function is not part of POSIX and therefore no official
+   cancellation point.  But due to similarity with an POSIX interface
+   or due to the implementation it is a cancellation point and
+   therefore not marked with __THROW.  */
 extern int rcmd_af (char **__restrict __ahost, unsigned short int __rport,
 		    __const char *__restrict __locuser,
 		    __const char *__restrict __remuser,
 		    __const char *__restrict __cmd, int *__restrict __fd2p,
-		    sa_family_t __af) __THROW;
+		    sa_family_t __af);
 
 /* Call `rexecd' at port RPORT on remote machine *AHOST to execute
    CMD.  The process runs at the remote machine using the ID of user
    NAME whose cleartext password is PASSWD.  In *FD2P the descriptor
    to the socket for the connection is returned.  When the function
-   returns *AHOST contains the official host name.  */
+   returns *AHOST contains the official host name.
+
+   This function is not part of POSIX and therefore no official
+   cancellation point.  But due to similarity with an POSIX interface
+   or due to the implementation it is a cancellation point and
+   therefore not marked with __THROW.  */
 extern int rexec (char **__restrict __ahost, int __rport,
 		  __const char *__restrict __name,
 		  __const char *__restrict __pass,
-		  __const char *__restrict __cmd, int *__restrict __fd2p)
-     __THROW;
+		  __const char *__restrict __cmd, int *__restrict __fd2p);
 
 /* This is the equivalent function where the protocol can be selected
-   and which therefore can be used for IPv6.  */
+   and which therefore can be used for IPv6.
+
+   This function is not part of POSIX and therefore no official
+   cancellation point.  But due to similarity with an POSIX interface
+   or due to the implementation it is a cancellation point and
+   therefore not marked with __THROW.  */
 extern int rexec_af (char **__restrict __ahost, int __rport,
 		     __const char *__restrict __name,
 		     __const char *__restrict __pass,
 		     __const char *__restrict __cmd, int *__restrict __fd2p,
-		     sa_family_t __af) __THROW;
+		     sa_family_t __af);
 
 /* Check whether user REMUSER on system RHOST is allowed to login as LOCUSER.
    If SUSER is not zero the user tries to become superuser.  Return 0 if
-   it is possible.  */
+   it is possible.
+
+   This function is not part of POSIX and therefore no official
+   cancellation point.  But due to similarity with an POSIX interface
+   or due to the implementation it is a cancellation point and
+   therefore not marked with __THROW.  */
 extern int ruserok (__const char *__rhost, int __suser,
-		    __const char *__remuser, __const char *__locuser) __THROW;
+		    __const char *__remuser, __const char *__locuser);
 
 /* This is the equivalent function where the protocol can be selected
-   and which therefore can be used for IPv6.  */
+   and which therefore can be used for IPv6.
+
+   This function is not part of POSIX and therefore no official
+   cancellation point.  But due to similarity with an POSIX interface
+   or due to the implementation it is a cancellation point and
+   therefore not marked with __THROW.  */
 extern int ruserok_af (__const char *__rhost, int __suser,
 		       __const char *__remuser, __const char *__locuser,
-		       sa_family_t __af) __THROW;
+		       sa_family_t __af);
 
 /* Try to allocate reserved port, returning a descriptor for a socket opened
    at this port or -1 if unsuccessful.  The search for an available port
-   will start at ALPORT and continues with lower numbers.  */
-extern int rresvport (int *__alport) __THROW;
+   will start at ALPORT and continues with lower numbers.
+
+   This function is not part of POSIX and therefore no official
+   cancellation point.  But due to similarity with an POSIX interface
+   or due to the implementation it is a cancellation point and
+   therefore not marked with __THROW.  */
+extern int rresvport (int *__alport);
 
 /* This is the equivalent function where the protocol can be selected
-   and which therefore can be used for IPv6.  */
-extern int rresvport_af (int *__alport, sa_family_t __af) __THROW;
+   and which therefore can be used for IPv6.
+
+   This function is not part of POSIX and therefore no official
+   cancellation point.  But due to similarity with an POSIX interface
+   or due to the implementation it is a cancellation point and
+   therefore not marked with __THROW.  */
+extern int rresvport_af (int *__alport, sa_family_t __af);
 #endif
 
 
@@ -424,9 +569,13 @@ struct gaicb
 # define AI_PASSIVE	0x0001	/* Socket address is intended for `bind'.  */
 # define AI_CANONNAME	0x0002	/* Request for canonical name.  */
 # define AI_NUMERICHOST	0x0004	/* Don't use name resolution.  */
+# define AI_V4MAPPED	0x0008	/* IPv4 mapped addresses are acceptable.  */
+# define AI_ALL		0x0010	/* Return IPv4 mapped and IPv6 addresses.  */
+# define AI_ADDRCONFIG	0x0020	/* Use configuration of this host to choose
+				   returned address type..  */
 # ifdef __USE_GNU
-#  define AI_IDN	0x0008	/* IDN encode input (assuming it is encoded
-                                   in the current locale's character set)
+#  define AI_IDN	0x0040	/* IDN encode input (assuming it is encoded
+				   in the current locale's character set)
 				   before looking it up. */
 # endif
 
@@ -448,7 +597,7 @@ struct gaicb
 #  define EAI_NOTCANCELED -102	/* Request not canceled.  */
 #  define EAI_ALLDONE	  -103	/* All requests done.  */
 #  define EAI_INTR	  -104	/* Interrupted by a signal.  */
-#  define EAI_IDN_ENCODE  -105	/* IDN encoding of input failed.  */
+#  define EAI_IDN_ENCODE  -105	/* IDN encoding failed.  */
 # endif
 
 # define NI_MAXHOST      1025
@@ -461,11 +610,14 @@ struct gaicb
 # define NI_DGRAM	16	/* Look up UDP service rather than TCP.  */
 
 /* Translate name of a service location and/or a service name to set of
-   socket addresses.  */
+   socket addresses.
+
+   This function is a possible cancellation point and therefore not
+   marked with __THROW.  */
 extern int getaddrinfo (__const char *__restrict __name,
 			__const char *__restrict __service,
 			__const struct addrinfo *__restrict __req,
-			struct addrinfo **__restrict __pai) __THROW;
+			struct addrinfo **__restrict __pai);
 
 /* Free `addrinfo' structure AI including associated storage.  */
 extern void freeaddrinfo (struct addrinfo *__ai) __THROW;
@@ -473,33 +625,45 @@ extern void freeaddrinfo (struct addrinfo *__ai) __THROW;
 /* Convert error return from getaddrinfo() to a string.  */
 extern __const char *gai_strerror (int __ecode) __THROW;
 
-/* Translate a socket address to a location and service name.  */
+/* Translate a socket address to a location and service name.
+
+   This function is a possible cancellation point and therefore not
+   marked with __THROW.  */
 extern int getnameinfo (__const struct sockaddr *__restrict __sa,
 			socklen_t __salen, char *__restrict __host,
 			socklen_t __hostlen, char *__restrict __serv,
-			socklen_t __servlen, unsigned int __flags) __THROW;
+			socklen_t __servlen, unsigned int __flags);
+#endif	/* POSIX */
 
-# ifdef __USE_GNU
+#ifdef __USE_GNU
 /* Enqueue ENT requests from the LIST.  If MODE is GAI_WAIT wait until all
    requests are handled.  If WAIT is GAI_NOWAIT return immediately after
-   queueing the requests and signal completion according to SIG.  */
+   queueing the requests and signal completion according to SIG.
+
+   This function is not part of POSIX and therefore no official
+   cancellation point.  But due to similarity with an POSIX interface
+   or due to the implementation it is a cancellation point and
+   therefore not marked with __THROW.  */
 extern int getaddrinfo_a (int __mode, struct gaicb *__list[__restrict_arr],
-			  int __ent, struct sigevent *__restrict __sig)
-     __THROW;
+			  int __ent, struct sigevent *__restrict __sig);
 
 /* Suspend execution of the thread until at least one of the ENT requests
    in LIST is handled.  If TIMEOUT is not a null pointer it specifies the
-   longest time the function keeps waiting before returning with an error.  */
+   longest time the function keeps waiting before returning with an error.
+
+   This function is not part of POSIX and therefore no official
+   cancellation point.  But due to similarity with an POSIX interface
+   or due to the implementation it is a cancellation point and
+   therefore not marked with __THROW.  */
 extern int gai_suspend (__const struct gaicb *__const __list[], int __ent,
-			__const struct timespec *__timeout) __THROW;
+			__const struct timespec *__timeout);
 
 /* Get the error status of the request REQ.  */
 extern int gai_error (struct gaicb *__req) __THROW;
 
 /* Cancel the requests associated with GAICBP.  */
 extern int gai_cancel (struct gaicb *__gaicbp) __THROW;
-# endif	/* GNU */
-#endif	/* POSIX */
+#endif	/* GNU */
 
 __END_DECLS
 
